# Generated by Django 4.1.3 on 2022-12-17 00:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0003_rule_action'),
    ]

    operations = [
        migrations.AddField(
            model_name='serverossecalert',
            name='ssh_sig',
            field=models.TextField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name='rulecheck',
            name='parent',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='checks', to='incident.rule'),
        ),
    ]
