# Generated by Django 4.1.3 on 2022-12-17 02:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0004_serverossecalert_ssh_sig_alter_rulecheck_parent'),
    ]

    operations = [
        migrations.AddField(
            model_name='incident',
            name='component',
            field=models.CharField(db_index=True, default=None, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='incident',
            name='state',
            field=models.IntegerField(choices=[(0, 'new'), (1, 'opened'), (2, 'paused'), (3, 'ignored'), (4, 'resolved')], default=0),
        ),
    ]
