# Generated by Django 4.1.4 on 2023-01-23 05:04

from django.db import migrations, models
import django.db.models.deletion
import rest.models


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0012_settings_settingsmetadata'),
        ('inbox', '0003_attachment_content_type'),
    ]

    operations = [
        migrations.CreateModel(
            name='MailTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(db_index=True, max_length=255)),
                ('kind', models.CharField(default='email', max_length=124)),
                ('template', models.TextField()),
                ('group', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='templates', to='account.group')),
            ],
            bases=(models.Model, rest.models.RestModel),
        ),
    ]
