# Generated by Django 4.0.3 on 2022-03-29 19:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0004_group_modified_alter_group_created'),
    ]

    operations = [
        migrations.AddField(
            model_name='member',
            name='auth_code_expires',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
    ]
