# Generated by Django 4.1.4 on 2023-07-13 15:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0012_settings_settingsmetadata'),
    ]

    operations = [
        migrations.AddField(
            model_name='memberdevice',
            name='ip',
            field=models.CharField(default=None, max_length=64, null=True),
        ),
    ]
